package tech.nevets.jaml.gui;

import tech.nevets.jaml.Config;
import tech.nevets.jaml.JAML;
import tech.nevets.jaml.gui.panels.CustomProgressBarPanel;
import tech.nevets.jaml.util.ImageUtils;
import tech.nevets.jaml.util.LoaderUtils;
import tech.nevets.jaml.util.ProfileUtils;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.nio.file.Path;

public class StartupGui extends JFrame {
    JPanel contentPane;
    private static JLabel currentTaskLabel;

    public StartupGui() {
        setUndecorated(true);
        setResizable(false);
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        setIconImage(new ImageIcon(getClass().getClassLoader().getResource("assets/icon.png")).getImage());
        setSize(GuiHandler.SCREEN_WIDTH / 3, GuiHandler.SCREEN_HEIGHT / 3);
        contentPane = new JPanel();
        contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
        setContentPane(contentPane);
        contentPane.setLayout(null);
        setLocationRelativeTo(null);

        currentTaskLabel = new JLabel("");
        currentTaskLabel.setBounds(30, 279, 176, 20);
        contentPane.add(currentTaskLabel);

        int pbWidth = (int) (getWidth() / 1.1);
        int pbHeight = getHeight() / 24;
        int pbX = (getWidth() - pbWidth) / 2;
        int pbY = getHeight() - (pbHeight * 2);

        JLabel progressBarOutline = new JLabel();
        progressBarOutline.setBounds(pbX, pbY - 10, pbWidth, pbHeight + 20);
        ImageIcon pgOutline = new ImageIcon(getClass().getClassLoader().getResource("assets/progressbaroutline.png"));
        pgOutline = ImageUtils.resizeIcon(pgOutline, pbWidth + 20, pbHeight);
        progressBarOutline.setIcon(pgOutline);
        contentPane.add(progressBarOutline);

        CustomProgressBarPanel progressBar = new CustomProgressBarPanel(pbWidth, pbHeight);
        progressBar.setBounds(pbX, pbY, pbWidth, pbHeight);
        contentPane.add(progressBar);
        
        JLabel splashIcon = new JLabel("");
        splashIcon.setSize(getWidth(), getHeight());
        ImageIcon backgroundImg = new ImageIcon(getClass().getClassLoader().getResource("assets/background.png"));
        backgroundImg = ImageUtils.resizeIcon(backgroundImg, splashIcon.getWidth(), splashIcon.getHeight());
        splashIcon.setIcon(backgroundImg);
        contentPane.add(splashIcon);

        setVisible(true);


        progressBar.setProgress(0);
        currentTaskLabel.setText("Making Directories...");
        makeDirs();
        smoothIncrease(progressBar, 20);
        currentTaskLabel.setText("Loading Config...");
        Config.loadConfig();
        smoothIncrease(progressBar, 40);
        currentTaskLabel.setText("Loading Profiles...");
        ProfileUtils.getProfileList();
        smoothIncrease(progressBar, 60);
        currentTaskLabel.setText("Loading Active Profile...");
        ProfileUtils.getActiveProfile();
        smoothIncrease(progressBar, 80);
        currentTaskLabel.setText("Getting Game Loaders...");
        LoaderUtils.getLoaders();
        currentTaskLabel.setText("Done!");
        smoothIncrease(progressBar, 100);

    }

    private static void sleep() {
        try {
            Thread.sleep(500);
        } catch (InterruptedException ignored) {

        }
    }

    public static void setCurrentTask(String currentTask) {
        currentTaskLabel.setText(currentTask);
    }

    private static void makeDirs() {
        if (System.getenv("JAML_HOME") == null) {
            JAML.path = Path.of(System.getenv("APPDATA") + "\\.jaml\\");
        } else {
            JAML.path = Path.of(System.getenv("JAML_HOME"));
        }

        try {
            if (!JAML.path.toFile().exists()) {
                JAML.path.toFile().mkdirs();
            }
        } catch (Exception e) {
            System.out.println("Invalid path, please check your config.yml and try again");
        }

        try {
            Path profilePath = Path.of(JAML.path + "\\profiles\\");
            if (!profilePath.toFile().exists()) {
                profilePath.toFile().mkdirs();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        try {
            Path profilePath = Path.of(JAML.path + "\\assets\\");
            if (!profilePath.toFile().exists()) {
                profilePath.toFile().mkdirs();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        try {
            Path dataPath = Path.of(JAML.path + "\\data\\");
            if (!dataPath.toFile().exists()) {
                dataPath.toFile().mkdirs();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        try {
            Path skinsPath = Path.of(JAML.path + "\\data\\skins\\");
            if (!skinsPath.toFile().exists()) {
                skinsPath.toFile().mkdirs();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private static void smoothIncrease(CustomProgressBarPanel pb, int to) {
        int i = pb.getProgress();
        while (i <= to) {
            pb.setProgress(i);
            i += 1;
        }
    }
}
